clc;clear all;close all;
% First, specify the root directory where the Tensor Toolbox and
% N-way Toolbox folders can be found. The code has been tested using
% Tensor Toolbox v2.6 and N-Way Toolbox v3.30.
ROOTPATH = '';
addpath(strcat(ROOTPATH, 'tensor_toolbox'));
addpath(strcat(ROOTPATH, 'nway'));
delete(gcp('nocreate'));

%% create parafac2 problem 
K = 1000; %number of subjects (K)
I = 100; %max number of variables (I_k)-> varies
J = 500; %number of observations
sparsity = 10^-3; %sparsity of each input matrix X_k
F = 5; %target rank 
c = parcluster('local'); % build the 'local' cluster object
nw = c.NumWorkers  ;      % get the number of workers
PARFOR_FLAG=1;  if PARFOR_FLAG==1; parpool('local',nw);end

[X, totalnnz] = create_parafac2_problem(K, J, F, sparsity, I, PARFOR_FLAG);
strFile=strcat('syntheticData/X_',int2str(K),'_',int2str(J),'_',int2str(I),'_',int2str(totalnnz),'_File.mat');
save(strFile,'X','totalnnz');

%% online batch settings

ratio=0.1; minibatchSize=round(ratio*0.5*K);
[Xold, dimsX, initialBatch, streamingBatch, idx]=getDataforStreaming(X,ratio);

%% initialization
st=tic;
[Ls, Ms, Facs, Pold, Uold] = spade_initial( Xold,F,PARFOR_FLAG);
Cold=Facs{3};
disp('********** initialization done **********');

%% adding new data
for t=1:minibatchSize:streamingBatch
    fprintf('Processing %d\n',t);
    endTime = min(initialBatch+t+minibatchSize-1, dimsX(end));
    idx(end) = {initialBatch+t:endTime};
    newX = squeeze(X(idx{:}));
    numOfSlice = endTime-initialBatch-t+1;
    
    [Bnew, Hnew, Cnew,  Unew, Ls, Ms]=spade_update( newX,F, Facs, Ls, Ms );
    % update temporal data
     Cold=[Cold;Cnew];
    % update U matrices
    Uold=[Uold Unew];
    
    Facs{1}=Hnew;Facs{2}=Bnew;Facs{3}=Cold;
end

 time_spade=toc(st);
 disp('Compute fitness');
 [fit_spade]=pf2fitout(X,Uold,Bnew,Cold,K);

%% baseline 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Baseline 1: PARAFC2');
ALG = 0;
if (ALG==1)
    disp('SPARTan execution');  % if ALG==1, then SPARTan is executed. 
else
    disp('Sparse PARAFAC2 baseline execution'); % if ALG==0, then the baseline is executed
end
Maxiters = 1000; % maximum number of iterations
Constraints = [0 0]; % non-negative constraints on both the V and S_k factors
Options = [1e-6 Maxiters 2 0 0 ALG PARFOR_FLAG]; % first argument is convergence criterion


[H_parafac2, B_parafac2, C_parafac2, P_parafac2,~, ~, ~]=parafac2_sparse_paper_version(X,F,Constraints,Options);
Unew_parafac2=cell(1,K);
for k=1:K
        Unew_parafac2{k}=P_parafac2{k}*H_parafac2;
end
time_parafac2=toc(st);
[fit_parafac2]=pf2fitout(X,Unew_parafac2,B_parafac2, C_parafac2, K);
%% baseline2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Baseline 2: Sparse PARAFAC2');
ALG = 1;
if (ALG==1)
    disp('SPARTan execution');  % if ALG==1, then SPARTan is executed. 
else
    disp('Sparse PARAFAC2 baseline execution'); % if ALG==0, then the baseline is executed
end
Maxiters = 1000; % maximum number of iterations
Constraints = [0 0]; % non-negative constraints on both the V and S_k factors
Options = [1e-6 Maxiters 2 0 0 ALG PARFOR_FLAG]; % first argument is convergence criterion

 st=tic;
[H_spartan, B_spartan, C_spartan, P_spartan, ~, ~,~]=parafac2_sparse_paper_version(X,F,Constraints,Options);
Unew_spartan=cell(1,K);
for k=1:K
        Unew_spartan{k}=P_spartan{k}*H_spartan;
end
time_spartan=toc(st);
[fit_spartan]=pf2fitout(X,Unew_spartan,B_spartan, C_spartan, K);
%% Results
fprintf('Method \t LOSS \t Time \n')
fprintf('SPADE \t %f \t %f \n',fit_spade,time_spade);
fprintf('PARAFAC2 \t %f \t %f \n',fit_parafac2,time_parafac2);
fprintf('SPARTAN \t %f \t %f \n',fit_spartan,time_spartan);

delete(gcp('nocreate'))
